searchData={"items":[{"type":"module","title":"Abit","doc":"Abit let's you use `:atomics` as a bit array or as an array of N-bit counters.\n\nTo know more about `:atomics` check the [Erlang atomics documentation](http://erlang.org/doc/man/atomics.html)\n\nThe `Abit` module (this module) has functions to use `:atomics` as a bit array.\nThe bit array is zero indexed.\n\nThe `Abit.Counter` module has functions to use `:atomics` as an array of N-bit\ncounters.\n\nThe functions in `Abit.Bitmask` help working with bitmasks.","ref":"Abit.html"},{"type":"module","title":"Abit - Abit","doc":"iex> ref = :atomics.new(100, signed: false)\n    iex> Abit.bit_count(ref)\n    6400\n    iex> Abit.bit_at(ref, 0)\n    0\n    iex> Abit.set_bit_at(ref, 0, 1)\n    :ok\n    iex> Abit.bit_at(ref, 0)\n    1","ref":"Abit.html#module-abit"},{"type":"module","title":"Abit.Counter - Abit","doc":"iex> counter = %Abit.Counter{} = Abit.Counter.new(100, 16)\n    iex> Abit.Counter.get(counter, 0)\n    0\n    iex> Abit.Counter.put(counter, 0, 100)\n    {:ok, {0, 100}}\n    iex> Abit.Counter.add(counter, 0, 100)\n    {:ok, {0, 200}}\n    iex> Abit.Counter.member?(counter, 200)\n    true","ref":"Abit.html#module-abit-counter"},{"type":"module","title":"Abit.Bitmask - Abit","doc":"iex> Abit.Bitmask.set_bits_count(3)\n    2\n    iex> Abit.Bitmask.bit_at(2, 0)\n    0\n    iex> Abit.Bitmask.bit_at(2, 1)\n    1","ref":"Abit.html#module-abit-bitmask"},{"type":"function","title":"Abit.bit_at/2","doc":"Returns bit at `bit_index` in atomic `ref`.","ref":"Abit.html#bit_at/2"},{"type":"function","title":"Examples - Abit.bit_at/2","doc":"iex> ref = :atomics.new(1, signed: false)\n    iex> ref |> :atomics.put(1, 3)\n    iex> Abit.bit_at(ref, 0)\n    1\n    iex> Abit.bit_at(ref, 1)\n    1\n    iex> Abit.bit_at(ref, 2)\n    0","ref":"Abit.html#bit_at/2-examples"},{"type":"function","title":"Abit.bit_count/1","doc":"Returns total count of bits in atomics `ref`.\n\n`:atomics` are 64 bit integers so total\ncount of bits is size * 64.","ref":"Abit.html#bit_count/1"},{"type":"function","title":"Examples - Abit.bit_count/1","doc":"iex> ref = :atomics.new(1, signed: false)\n    iex> Abit.bit_count(ref)\n    64\n    iex> ref2 = :atomics.new(2, signed: false)\n    iex> Abit.bit_count(ref2)\n    128","ref":"Abit.html#bit_count/1-examples"},{"type":"function","title":"Abit.bit_position/1","doc":"Returns position of bit in `:atomics`.\n\nReturns a 2 tuple containing:\n  * `atomics_index` - the index of the integer in atomics where the bit is located\n  * `bit_index` - the index of the bit in the integer","ref":"Abit.html#bit_position/1"},{"type":"function","title":"Examples - Abit.bit_position/1","doc":"iex> Abit.bit_position(0)\n    {1, 0}\n    iex> Abit.bit_position(11)\n    {1, 11}\n    iex> Abit.bit_position(64)\n    {2, 0}","ref":"Abit.html#bit_position/1-examples"},{"type":"function","title":"Abit.hamming_distance/2","doc":"Returns the bitwise Hamming distance between the two\ngiven `:atomics` references `ref_l` and `ref_r`.\n\nRaises ArgumentError if the sizes of `ref_l` and `ref_r` are not equal.","ref":"Abit.html#hamming_distance/2"},{"type":"function","title":"Examples - Abit.hamming_distance/2","doc":"iex> ref_l = :atomics.new(10, signed: false)\n    iex> ref_r = :atomics.new(10, signed: false)\n    iex> Abit.hamming_distance(ref_l, ref_r)\n    0\n    iex> ref_l |> :atomics.put(1, 7)\n    iex> Abit.hamming_distance(ref_l, ref_r)\n    3","ref":"Abit.html#hamming_distance/2-examples"},{"type":"function","title":"Abit.intersect/2","doc":"Bit intersection of atomics using Bitwise AND operator.\n\nReturns `ref_a` mutated.","ref":"Abit.html#intersect/2"},{"type":"function","title":"Abit.merge/2","doc":"Merge bits of atomics `ref_a` & `ref_b` using the\nbitwise OR operator.\n\n`ref_b` will be merged into `ref_a`.\nReturns `ref_a` mutated.","ref":"Abit.html#merge/2"},{"type":"function","title":"Abit.set_bit_at/3","doc":"Sets the bit at `bit_index` to `bit` in the atomics `ref`.\n\nReturns `:ok`.","ref":"Abit.html#set_bit_at/3"},{"type":"function","title":"Examples - Abit.set_bit_at/3","doc":"iex> ref = :atomics.new(1, signed: false)\n    iex> ref |> Abit.set_bit_at(0, 1)\n    iex> ref |> :atomics.get(1)\n    1\n    iex> ref |> Abit.set_bit_at(0, 0)\n    :ok\n    iex> ref |> :atomics.get(1)\n    0","ref":"Abit.html#set_bit_at/3-examples"},{"type":"function","title":"Abit.set_bits_count/1","doc":"Returns number of bits set to 1 in atomics array `ref`.","ref":"Abit.html#set_bits_count/1"},{"type":"function","title":"Examples - Abit.set_bits_count/1","doc":"iex> ref = :atomics.new(1, signed: false)\n    iex> ref |> :atomics.put(1, 3)\n    iex> Abit.set_bits_count(ref)\n    2\n    iex> ref2 = :atomics.new(1, signed: false)\n    iex> Abit.set_bits_count(ref2)\n    0","ref":"Abit.html#set_bits_count/1-examples"},{"type":"function","title":"Abit.to_list/1","doc":"Returns a flat list of every atomic value converted\ninto a list of bits from `ref` atomics reference.","ref":"Abit.html#to_list/1"},{"type":"function","title":"Examples - Abit.to_list/1","doc":"ref = :atomics.new(10, signed: false)\n    ref |> Abit.to_list\n    [0, 0, 0, 0, 0, ...]","ref":"Abit.html#to_list/1-examples"},{"type":"module","title":"Abit.Atomics","doc":"This module provides utility functions for working with Erlang's :atomics.","ref":"Abit.Atomics.html"},{"type":"function","title":"Abit.Atomics.deserialize/1","doc":"Deserializes a binary into an :atomics reference.\n\nThis function takes a binary that was previously serialized using `serialize/1`\nand reconstructs an :atomics reference from it.","ref":"Abit.Atomics.html#deserialize/1"},{"type":"function","title":"Parameters - Abit.Atomics.deserialize/1","doc":"* `binary` - A binary containing the serialized :atomics data.","ref":"Abit.Atomics.html#deserialize/1-parameters"},{"type":"function","title":"Returns - Abit.Atomics.deserialize/1","doc":"A reference to a new :atomics array containing the deserialized data.","ref":"Abit.Atomics.html#deserialize/1-returns"},{"type":"function","title":"Examples - Abit.Atomics.deserialize/1","doc":"iex> ref = :atomics.new(2, [])\n    iex> :atomics.put(ref, 1, 10)\n    iex> :atomics.put(ref, 2, -20)\n    iex> serialized = Abit.Atomics.serialize(ref)\n    iex> deserialized_ref = Abit.Atomics.deserialize(serialized)\n    iex> Abit.Atomics.to_list(deserialized_ref)\n    [10, -20]","ref":"Abit.Atomics.html#deserialize/1-examples"},{"type":"function","title":"Abit.Atomics.member?/2","doc":"Checks if an integer is present in the :atomics reference.\n\nThis function checks if the given integer exists as a value in any of the\natomics within the provided :atomics reference.","ref":"Abit.Atomics.html#member?/2"},{"type":"function","title":"Parameters - Abit.Atomics.member?/2","doc":"* `atomics_ref` - A reference to an :atomics array.\n  * `int` - The integer to search for.","ref":"Abit.Atomics.html#member?/2-parameters"},{"type":"function","title":"Returns - Abit.Atomics.member?/2","doc":"Returns `true` if the integer is found in the :atomics array, `false` otherwise.","ref":"Abit.Atomics.html#member?/2-returns"},{"type":"function","title":"Examples - Abit.Atomics.member?/2","doc":"iex> ref = :atomics.new(3, signed: false)\n    iex> :atomics.put(ref, 1, 10)\n    iex> :atomics.put(ref, 2, 20)\n    iex> :atomics.put(ref, 3, 30)\n    iex> Abit.Atomics.member?(ref, 20)\n    true\n    iex> Abit.Atomics.member?(ref, 40)\n    false","ref":"Abit.Atomics.html#member?/2-examples"},{"type":"function","title":"Abit.Atomics.serialize/1","doc":"Serializes an :atomics reference into a binary.\n\nThis function takes an :atomics reference and returns a binary where each\n64-bit integer in the :atomics array is encoded in big-endian format.","ref":"Abit.Atomics.html#serialize/1"},{"type":"function","title":"Parameters - Abit.Atomics.serialize/1","doc":"* `atomics_ref` - A reference to an :atomics array.","ref":"Abit.Atomics.html#serialize/1-parameters"},{"type":"function","title":"Returns - Abit.Atomics.serialize/1","doc":"A binary containing the serialized :atomics data.","ref":"Abit.Atomics.html#serialize/1-returns"},{"type":"function","title":"Examples - Abit.Atomics.serialize/1","doc":"iex> ref = :atomics.new(2, signed: false)\n    iex> :atomics.put(ref, 1, 1)\n    iex> :atomics.put(ref, 2, 2)\n    iex> Abit.Atomics.serialize(ref)\n    <<0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 2>>","ref":"Abit.Atomics.html#serialize/1-examples"},{"type":"function","title":"Abit.Atomics.to_list/1","doc":"Converts an :atomics reference to a list of integers.\n\nThis function takes an :atomics reference and returns a list of integers,\nwhere each integer represents the value stored in each atomic.","ref":"Abit.Atomics.html#to_list/1"},{"type":"function","title":"Parameters - Abit.Atomics.to_list/1","doc":"* `atomics_ref` - A reference to an :atomics array.","ref":"Abit.Atomics.html#to_list/1-parameters"},{"type":"function","title":"Returns - Abit.Atomics.to_list/1","doc":"A list of integers representing the values stored in the :atomics array.","ref":"Abit.Atomics.html#to_list/1-returns"},{"type":"function","title":"Examples - Abit.Atomics.to_list/1","doc":"iex> ref = :atomics.new(3, signed: false)\n    iex> :atomics.put(ref, 1, 10)\n    iex> :atomics.put(ref, 2, 20)\n    iex> :atomics.put(ref, 3, 30)\n    iex> Abit.Atomics.to_list(ref)\n    [10, 20, 30]","ref":"Abit.Atomics.html#to_list/1-examples"},{"type":"module","title":"Abit.Bitmask","doc":"Functions for working with bits & integer bitmasks.","ref":"Abit.Bitmask.html"},{"type":"function","title":"Abit.Bitmask.bit_at/2","doc":"Returns bit at `bit_index` in the given `integer`.","ref":"Abit.Bitmask.html#bit_at/2"},{"type":"function","title":"Examples - Abit.Bitmask.bit_at/2","doc":"iex> Abit.Bitmask.bit_at(2, 0)\n    0\n    iex> Abit.Bitmask.bit_at(2, 1)\n    1\n    iex> Abit.Bitmask.bit_at(1, 0)\n    1\n    iex> Abit.Bitmask.bit_at(0, 0)\n    0","ref":"Abit.Bitmask.html#bit_at/2-examples"},{"type":"function","title":"Abit.Bitmask.hamming_distance/2","doc":"Returns the bitwise Hamming distance between the\ngiven integers `int_l` and `int_r`.","ref":"Abit.Bitmask.html#hamming_distance/2"},{"type":"function","title":"Examples - Abit.Bitmask.hamming_distance/2","doc":"iex> Abit.Bitmask.hamming_distance(1, 1)\n    0\n    iex> Abit.Bitmask.hamming_distance(1, 0)\n    1\n    iex> Abit.Bitmask.hamming_distance(1, 1023)\n    9\n    iex> Abit.Bitmask.hamming_distance(1, 1024)\n    2","ref":"Abit.Bitmask.html#hamming_distance/2-examples"},{"type":"function","title":"Abit.Bitmask.set_bit_at/3","doc":"Sets the bit at `bit_index` in `integer` and returns it.","ref":"Abit.Bitmask.html#set_bit_at/3"},{"type":"function","title":"Examples - Abit.Bitmask.set_bit_at/3","doc":"iex> Abit.Bitmask.set_bit_at(1, 0, 0)\n    0\n    iex> Abit.Bitmask.set_bit_at(0, 0, 1)\n    1\n    iex> Abit.Bitmask.set_bit_at(0, 2, 1)\n    4","ref":"Abit.Bitmask.html#set_bit_at/3-examples"},{"type":"function","title":"Abit.Bitmask.set_bits_count/1","doc":"Returns the count of bits set to 1 in the given integer `int`.","ref":"Abit.Bitmask.html#set_bits_count/1"},{"type":"function","title":"Examples - Abit.Bitmask.set_bits_count/1","doc":"iex> Abit.Bitmask.set_bits_count(3)\n    2\n    iex> Abit.Bitmask.set_bits_count(0)\n    0\n    iex> Abit.Bitmask.set_bits_count(1024)\n    1\n    iex> Abit.Bitmask.set_bits_count(1023)\n    10","ref":"Abit.Bitmask.html#set_bits_count/1-examples"},{"type":"function","title":"Abit.Bitmask.to_list/2","doc":"Converts the given `integer` to a list of bits.\n\n`size` is the size of the bitstring you want the integer to be\nconverted to before creating a list from it.","ref":"Abit.Bitmask.html#to_list/2"},{"type":"function","title":"Examples - Abit.Bitmask.to_list/2","doc":"iex> Abit.Bitmask.to_list(1, 1)\n    [1]\n    iex> Abit.Bitmask.to_list(1, 2)\n    [0, 1]\n    iex> Abit.Bitmask.to_list(214311324231232211111, 64)\n    [1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1]","ref":"Abit.Bitmask.html#to_list/2-examples"},{"type":"module","title":"Abit.Counter","doc":"Use `:atomics` as an array of counters with N bits per counter.\nAn `:atomics` is an array of 64 bit integers so the possible counters are below:\n\nPossible counters:\n    bits | unsigned value range | signed value range\n    2    | 0..3                 | -2..1\n    4    | 0..15                | -8..7\n    8    | 0..255               | -128..127\n    16   | 0..65535             | -32768..32767\n    32   | 0..4294967295        | -2147483648..2147483647\n\nIf you need 64 bit counters use\n[Erlang counters](http://erlang.org/doc/man/counters.html)\n\nThe option `:wrap_around` is set to `false` by default. With these\nsmall-ish counters this is a safe default.\nWhen `:wrap_around` is `false` using `put/3` or `add/3` when the value\nwould be out of bounds the error tuple `{:error, :value_out_of_bounds}`\nwill be returned and the stored counter value will not change.\n\nWhile Erlang `:atomics` are 1 indexed, `Abit.Counter` counters are 0 indexed.","ref":"Abit.Counter.html"},{"type":"module","title":"Enumerable protocol - Abit.Counter","doc":"`Abit.Counter` implements the Enumerable protocol, so all Enum functions can be used:\n\n    iex> c = Abit.Counter.new(1000, 16, signed: false)\n    iex> c |> Abit.Counter.put(700, 54321)\n    iex> c |> Enum.max()\n    54321","ref":"Abit.Counter.html#module-enumerable-protocol"},{"type":"module","title":"Examples - Abit.Counter","doc":"iex> c = Abit.Counter.new(1000, 8, signed: false)\n    iex> c |> Abit.Counter.put(0, 100)\n    {:ok, {0, 100}}\n    iex> c |> Abit.Counter.add(0, 100)\n    {:ok, {0, 200}}\n    iex> c |> Abit.Counter.add(0, 100)\n    {:error, :value_out_of_bounds}","ref":"Abit.Counter.html#module-examples"},{"type":"function","title":"Abit.Counter.add/3","doc":"Increments the value of the counter at `index` with `incr`.\n\nReturns `{:ok, {index, final_value}}` or `{:error, :value_out_of_bounds}` if\noption `wrap_around` is `false` and value is out of bounds.","ref":"Abit.Counter.html#add/3"},{"type":"function","title":"Examples - Abit.Counter.add/3","doc":"iex> c = Abit.Counter.new(10, 8)\n    iex> c |> Abit.Counter.add(7, -12)\n    {:ok, {7, -12}}\n    iex> c |> Abit.Counter.add(7, 36)\n    {:ok, {7, 24}}\n    iex> c |> Abit.Counter.put(1, 1000)\n    {:error, :value_out_of_bounds}","ref":"Abit.Counter.html#add/3-examples"},{"type":"function","title":"Abit.Counter.get/2","doc":"Returns the value of counter at `index`.","ref":"Abit.Counter.html#get/2"},{"type":"function","title":"Examples - Abit.Counter.get/2","doc":"iex> c = Abit.Counter.new(10, 8)\n    iex> c |> Abit.Counter.get(7)\n    0","ref":"Abit.Counter.html#get/2-examples"},{"type":"function","title":"Abit.Counter.get_all_at_atomic/2","doc":"Returns all counters from atomics at index.\n\nIndex of atomics are one-based.","ref":"Abit.Counter.html#get_all_at_atomic/2"},{"type":"function","title":"Examples - Abit.Counter.get_all_at_atomic/2","doc":"iex> c = Abit.Counter.new(100, 8)\n    iex> c |> Abit.Counter.put(3, -70)\n    iex> c |> Abit.Counter.get_all_at_atomic(1)\n    [0, 0, 0, 0, -70, 0, 0, 0]","ref":"Abit.Counter.html#get_all_at_atomic/2-examples"},{"type":"function","title":"Abit.Counter.member?/2","doc":"Returns `true` if any counter has the value `integer`,\n`false` otherwise.","ref":"Abit.Counter.html#member?/2"},{"type":"function","title":"Examples - Abit.Counter.member?/2","doc":"iex> c = Abit.Counter.new(100, 8)\n    iex> c |> Abit.Counter.member?(0)\n    true\n    iex> c |> Abit.Counter.member?(80)\n    false","ref":"Abit.Counter.html#member?/2-examples"},{"type":"function","title":"Abit.Counter.new/3","doc":"Returns a new `%Abit.Counter{}` struct.\n\n  * `size` - minimum number of counters to have, counters will fully fill the `:atomics`.\n    Check the `:size` key in the returned `%Abit.Counter{}` for the exact number of counters\n  * `counters_bit_size` - how many bits a counter should use","ref":"Abit.Counter.html#new/3"},{"type":"function","title":"Options - Abit.Counter.new/3","doc":"* `:signed` - whether to have signed or unsigned counters. Defaults to `true`.\n  * `:wrap_around` - whether counters should wrap around. Defaults to `false`.","ref":"Abit.Counter.html#new/3-options"},{"type":"function","title":"Examples - Abit.Counter.new/3","doc":"Abit.Counter.new(100, 8) # minimum 100 counters; 8 bits signed\n    Abit.Counter.new(10_000, 16, signed: false) # minimum 10_000 counters; 16 bits unsigned\n    Abit.Counter.new(10_000, 16, wrap_around: false) # don't wrap around","ref":"Abit.Counter.html#new/3-examples"},{"type":"function","title":"Abit.Counter.put/3","doc":"Puts the value into the counter at `index`.\n\nReturns `{:ok, {index, final_value}}` or `{:error, :value_out_of_bounds}` if\noption `wrap_around` is `false` and value is out of bounds.","ref":"Abit.Counter.html#put/3"},{"type":"function","title":"Examples - Abit.Counter.put/3","doc":"iex> c = Abit.Counter.new(10, 8)\n    iex> c |> Abit.Counter.put(7, -12)\n    {:ok, {7, -12}}\n    iex> c |> Abit.Counter.get(7)\n    -12\n    iex> c |> Abit.Counter.put(7, 128)\n    {:error, :value_out_of_bounds}","ref":"Abit.Counter.html#put/3-examples"},{"type":"type","title":"Abit.Counter.t/0","doc":"","ref":"Abit.Counter.html#t:t/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}